@@ -158,10 +158,18 @@ func MergedGlobalWorkloadsCollection(
 	// More setup to do here so we can't use nestedCollectionFromLocalAndRemote
 	LocalWorkloadServicesNamespaceIndex := krt.NewNamespaceIndex(localWorkloadServices)
 	LocalEndpointSlicesByIPIndex := endpointSliceAddressIndex(localCluster.EndpointSlices())
+	localClusterGetter := func(_ krt.HandlerContext) cluster.ID {
+		return localCluster.ID
+	}
+	localNetworkGetter := func(ctx krt.HandlerContext) network.ID {
+		nw := ptr.OrEmpty(krt.FetchOne(ctx, globalNetworks.LocalSystemNamespace.AsCollection()))
+		return nw.Network
+	}
 	LocalPodWorkloads := krt.NewCollection(
 		localCluster.Pods(),
 		podWorkloadBuilder(
 			meshConfig,
+			localNetworkGetter,
 			localAuthorizationPolicies,
 			localPeerAuths,
 			localWaypoints,
@@ -172,13 +180,8 @@ func MergedGlobalWorkloadsCollection(
 			localCluster.Namespaces(),
 			localNodeLocalities,
 			domainSuffix,
-			func(_ krt.HandlerContext) cluster.ID {
-				return localCluster.ID
-			},
-			func(ctx krt.HandlerContext) network.ID {
-				nw := ptr.OrEmpty(krt.FetchOne(ctx, globalNetworks.LocalSystemNamespace.AsCollection()))
-				return nw.Network
-			},
+			localClusterGetter,
+			localNetworkGetter,
 			globalNetworks.NetworkGateways,
 			globalNetworks.GatewaysByNetwork,
 			flags,
@@ -194,19 +197,15 @@ func MergedGlobalWorkloadsCollection(
 		workloadEntries,
 		workloadEntryWorkloadBuilder(
 			meshConfig,
+			localNetworkGetter,
 			localAuthorizationPolicies,
 			localPeerAuths,
 			localWaypoints,
 			localWorkloadServices,
 			LocalWorkloadServicesNamespaceIndex,
 			localCluster.Namespaces(),
-			func(_ krt.HandlerContext) cluster.ID {
-				return localCluster.ID
-			},
-			func(ctx krt.HandlerContext) network.ID {
-				nw := ptr.OrEmpty(krt.FetchOne(ctx, globalNetworks.LocalSystemNamespace.AsCollection()))
-				return nw.Network
-			},
+			localClusterGetter,
+			localNetworkGetter,
 			globalNetworks.NetworkGateways,
 			globalNetworks.GatewaysByNetwork,
 			flags,
@@ -228,13 +227,8 @@ func MergedGlobalWorkloadsCollection(
 			localPeerAuths,
 			localWaypoints,
 			localCluster.Namespaces(),
-			func(ctx krt.HandlerContext) cluster.ID {
-				return localCluster.ID
-			},
-			func(ctx krt.HandlerContext) network.ID {
-				nw := ptr.OrEmpty(krt.FetchOne(ctx, globalNetworks.LocalSystemNamespace.AsCollection()))
-				return nw.Network
-			},
+			localClusterGetter,
+			localNetworkGetter,
 			globalNetworks.NetworkGateways,
 			globalNetworks.GatewaysByNetwork,
 			flags,
@@ -256,13 +250,8 @@ func MergedGlobalWorkloadsCollection(
 		endpointSlicesBuilder(meshConfig,
 			localWorkloadServices,
 			domainSuffix,
-			func(ctx krt.HandlerContext) cluster.ID {
-				return localCluster.ID
-			},
-			func(ctx krt.HandlerContext) network.ID {
-				nw := ptr.OrEmpty(krt.FetchOne(ctx, globalNetworks.LocalSystemNamespace.AsCollection()))
-				return nw.Network
-			},
+			localClusterGetter,
+			localNetworkGetter,
 		),
 		opts.WithName("LocalEndpointSliceWorkloads")...,
 	)
@@ -415,6 +404,7 @@ func MergedGlobalWorkloadsCollection(
 				pods,
 				podWorkloadBuilder(
 					meshConfig,
+					localNetworkGetter,
 					localAuthorizationPolicies,
 					localPeerAuths,
 					waypoints,
@@ -465,6 +455,7 @@ func MergedGlobalWorkloadsCollection(
 				workloadEntries,
 				workloadEntryWorkloadBuilder(
 					meshConfig,
+					localNetworkGetter,
 					localAuthorizationPolicies,
 					localPeerAuths,
 					waypoints,
@@ -642,6 +633,7 @@ func MergedGlobalWorkloadsCollection(
 
 func workloadEntryWorkloadBuilder(
 	meshConfig krt.Singleton[MeshConfig],
+	localNetworkGetter func(krt.HandlerContext) network.ID,
 	authorizationPolicies krt.Collection[model.WorkloadAuthorization],
 	peerAuths krt.Collection[*securityclient.PeerAuthentication],
 	waypoints krt.Collection[Waypoint],
@@ -709,6 +701,17 @@ func workloadEntryWorkloadBuilder(
 		w.CanonicalName, w.CanonicalRevision = kubelabels.CanonicalService(wle.Labels, w.WorkloadName)
 
 		setTunnelProtocol(wle.Labels, wle.Annotations, w)
+		localNetwork := localNetworkGetter(ctx)
+		if network != localNetwork.String() {
+			// This is a remote workload that we'll never send directly; don't precompute
+			return &model.WorkloadInfo{
+				Workload:     w,
+				Labels:       wle.Labels,
+				Source:       kind.WorkloadEntry,
+				CreationTime: wle.CreationTimestamp.Time,
+			}
+		}
+
 		return precomputeWorkloadPtr(&model.WorkloadInfo{
 			Workload:     w,
 			Labels:       wle.Labels,
@@ -727,8 +730,12 @@ func (a *index) workloadEntryWorkloadBuilder(
 	workloadServicesNamespaceIndex krt.Index[string, model.ServiceInfo],
 	namespaces krt.Collection[*v1.Namespace],
 ) krt.TransformationSingle[*networkingclient.WorkloadEntry, model.WorkloadInfo] {
+	localNetworkGetter := func(ctx krt.HandlerContext) network.ID {
+		return a.Network(ctx)
+	}
 	return workloadEntryWorkloadBuilder(
 		meshConfig,
+		localNetworkGetter,
 		authorizationPolicies,
 		peerAuths,
 		waypoints,
@@ -738,9 +745,7 @@ func (a *index) workloadEntryWorkloadBuilder(
 		func(ctx krt.HandlerContext) cluster.ID {
 			return a.ClusterID
 		},
-		func(ctx krt.HandlerContext) network.ID {
-			return a.Network(ctx)
-		},
+		localNetworkGetter,
 		a.networks.NetworkGateways,
 		a.networks.GatewaysByNetwork,
 		a.Flags,
@@ -773,6 +778,7 @@ func computeWaypoint(
 
 func podWorkloadBuilder(
 	meshConfig krt.Singleton[MeshConfig],
+	localNetworkGetter func(krt.HandlerContext) network.ID,
 	authorizationPolicies krt.Collection[model.WorkloadAuthorization],
 	peerAuths krt.Collection[*securityclient.PeerAuthentication],
 	waypoints krt.Collection[Waypoint],
@@ -865,6 +871,16 @@ func podWorkloadBuilder(
 		w.CanonicalName, w.CanonicalRevision = kubelabels.CanonicalService(p.Labels, w.WorkloadName)
 
 		setTunnelProtocol(p.Labels, p.Annotations, w)
+		localNetwork := localNetworkGetter(ctx)
+		if network != localNetwork.String() {
+			// This is a remote workload that we'll never send directly; don't precompute
+			return &model.WorkloadInfo{
+				Workload:     w,
+				Labels:       p.Labels,
+				Source:       kind.Pod,
+				CreationTime: p.CreationTimestamp.Time,
+			}
+		}
 		return precomputeWorkloadPtr(&model.WorkloadInfo{
 			Workload:     w,
 			Labels:       p.Labels,
@@ -886,8 +902,12 @@ func (a *index) podWorkloadBuilder(
 	namespaces krt.Collection[*v1.Namespace],
 	nodes krt.Collection[Node],
 ) krt.TransformationSingle[*v1.Pod, model.WorkloadInfo] {
+	localNetworkGetter := func(ctx krt.HandlerContext) network.ID {
+		return a.Network(ctx)
+	}
 	return podWorkloadBuilder(
 		meshConfig,
+		localNetworkGetter,
 		authorizationPolicies,
 		peerAuths,
 		waypoints,
@@ -901,9 +921,7 @@ func (a *index) podWorkloadBuilder(
 		func(ctx krt.HandlerContext) cluster.ID {
 			return a.ClusterID
 		},
-		func(ctx krt.HandlerContext) network.ID {
-			return a.Network(ctx)
-		},
+		localNetworkGetter,
 		a.networks.NetworkGateways,
 		a.networks.GatewaysByNetwork,
 		a.Flags,
@@ -1352,14 +1370,21 @@ func constructServicesFromWorkloadEntry(p *networkingv1alpha3.WorkloadEntry, ser
 			// Named targetPort has different semantics from Service vs ServiceEntry
 			if svc.Source.Kind == kind.Service {
 				// Service has explicit named targetPorts.
-				if named, f := svc.PortNames[int32(port.ServicePort)]; f && named.TargetPortName != "" {
-					// This port is a named target port, look it up
-					tv, ok := p.Ports[named.TargetPortName]
-					if !ok {
-						// We needed an explicit port, but didn't find one - skip this port
-						continue
+				if named, f := svc.PortNames[int32(port.ServicePort)]; f {
+					if named.TargetPortName != "" {
+						// This port is a named target port, look it up
+						tv, ok := p.Ports[named.TargetPortName]
+						if !ok {
+							// We needed an explicit port, but didn't find one - skip this port
+							continue
+						}
+						targetPort = tv
+					} else {
+						tv, ok := p.Ports[named.PortName]
+						if ok {
+							targetPort = tv
+						}
 					}
-					targetPort = tv
 				}
 			} else {
 				// ServiceEntry has no explicit named targetPorts; targetPort only allows a number
